function PS311_Scheme_A(lambda, Tfinal)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function PS311_Scheme_A(lambda, Tfinal)
% ____________________________________________________________________________
% 18.311 (Principles of Applied Mathematics). MIT, Winter 2001. R. R. Rosales.
% ____________________________________________________________________________
%
% Demo to study how the numerical scheme (for the equation u_t + u_x = 0)
%
% u(n, j+1) = u(n, j) - lambda*(u(n+1, j) - u(n, j));  where  lambda = dt/dx;
%
% behaves.  Here u(n, j) denotes the value of the solution at x = x_n = n*dx,
% and t = t_j = j*dt.    The implementation uses periodic boundary conditions
% in 0 < x < 2*pi and 0 < t < Tfinal,
% with a "chopped" cosine initial condition: u(x, 0) = max(0, - cos(x)).
%
% The default for Tfinal is 1.00 (if not given as an input).
%
% This scheme is the one that appears in the problems
%    GBNS01 of Good and Bad Numerical Schemes,
%    vNSA01 of von Neumann Stability Analysis, and
%    AENS01 of Long wave equation associated to a numerical scheme.
%
% - THE CODE HAS BEEN WRITTEN SO THAT REPLACING THE INITIAL CONDITIONS,
% - AND OTHER SUCH THINGS, IS EASY.
%
% In problem GBNS01 you are asked to (numerically) study this scheme and
% decide if it is unstable or not. Specifically, for this scheme:
%
%   Is there some positive value of lambda (Lc > 0)  such that, for lambda
%   < Lc,  the solution of the scheme converges to the solution to the PDE
%   as dx ---> 0.  In this case the scheme is stable.
%
%   Else, you will find that   (as dx ---> 0)    the solutions develop very
%   large amplitude grid scale oscilations. Then the scheme is unstable.
%
%   If there is an Lc, find it (approximately) experimenting with the script.
%
%   WARNING: even a stable scheme blows up if lambda is in the unstable
%            range. Observing blow up for one lambda is not enough to
%   conclude instability. This does not mean you have to check arbitrarily
%   small lambda's --- since, typically, Lc is O(1).
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Length = 2*pi;% ---------- Period in space of the solutions.        See below.
Npoint = 50:50:200;% ----- Array with numerical grid sizes.         """"""""""
Freeze =  250;% ---------- If the errors grow, to the point where the solution
%                          reaches the size specified by Freeze,   we stop any
%                          further growth by scaling down to size Freeze at
%                          each time step. This avoids problems with plotting:
%                          axis limits over-flows.
if nargin < 2;
   Tfinal = 1.00; end;% -- Final time for numerical integration.    See below.
% ============================================================================
% Parameters for the numerical integration.
%     Length = period in space for solution: 0 <= x <= Length.
%     Tfinal = time for integration:         0 <= t <= Tfinal.
%     Npoint = positive integer array with the number of points in the space
%              grid.   Integrations will be done with dx = Length/Npoint(n),
%              with n=1 ... length(Npoint).
%     lambda = value given to the numerical parameter in the scheme.
%
% ============================================================================
% Below strings containing descriptions of:
%   (1) Equation solved, (2) Scheme used, (3) Numerical parameter(s).
%   These are used for
%   (A) Command line messages to the user [these should be in plain text].
%   (B) Titles and labels in the figures  [these should be in LaTeX format].
%
% --- Command line strings.
%
EQUATN = 'u_t + u_x = 0';
SCHEME = 'Explicit Forward Differences';
LAMBDA = 'dt/dx';
%
% --- LaTeX strings.
%
EQNtex = 'u_t + u_x = 0';
SCHtex = 'Explicit Forward Differences';
%
% ============================================================================
% Command line messages generated by demo, stating what it does.
%
fprintf( '\n Finite differences numerical scheme demo: PS311_Scheme_A.\n')
fprintf(['\n EQUATION: ', EQUATN, '.\n'])
fprintf(['\n SCHEME:   ', SCHEME, '.\n'])
fprintf( '\n CONSISTENCY AND STABILITY: ')
fprintf(['\n Is this scheme stable? ', ...
         'If so, for which values of LAMBDA = ', LAMBDA, '?'])
fprintf( '\n WARNING. If instabilities arise:  Once the errors grow large,')
fprintf( '\n          further growth is prevented by scaling the solution,')
fprintf( '\n          at each time step, down to a maximum fixed size.')
fprintf(['\n          Currently this size is ', num2str(Freeze), '.\n'])
%
% ============================================================================
% Request user input lambda if not in the script call.
%
if nargin < 1
   lambda = input(['  Enter the value of lambda = ', LAMBDA, ' = ']);
   if isempty(lambda); lambda = 1; end
   fprintf('\n')
end
if lambda <= 0; lambda = 1; end
%
% ============================================================================
% Define initial conditions (stored in u0) and integrate.
% Note that changing the initial conditions is very simple.
%
for n = 1:length(Npoint)
   N  = Npoint(n);   dx = Length/N;   x = Length*(1:N)/N;   dt = lambda*dx;
   u0 = max(0, - cos(x)) + (rand(1, N) - 0.5)/10^3;
   U  = u0;% --- U will have the solution, one time level per row.
   T  =  0;% --- T will have the times computed.
   t  = dt;% --- t will have the time to be computed next.
   up = u0;% --- In each time step, up contains the solution at the current
   un = u0;%     time, and the solution at the next time goes into un.
   %
   while t <= Tfinal + dt
      %
      % --- Here the scheme is implemented. The equation determining un is
      %
      un(1:(N-1)) = up(1:(N-1)) - lambda*(up(2:N) - up(1:(N-1)));
      un(N)       = up(N)       - lambda*(up(1)   - up(N));
      %
      % --- FREEZE ERROR GROWTH ONCE ERRORS GET LARGE ENOUGH.
      %     We do this by re-scaling solution to maximum allowed size.
      %
      uM = max(abs(un));
      if uM > Freeze; un = (Freeze/uM)*un; end
      %
      % --- Store updated solution at new time.
      %
      U = [U; un];   T = [T; t];   up = un;   t = t + dt;
   end
   zmax = max(max(U));
   zmin = min(min(U));
   MakeTheSchemeFigure(n, N)
   whitebg('k')
   mesh([0, x], T, [U(:, N), U]);
   hold on;   grid on;   rotate3d on
   axis([0 Length 0 (t-dt) zmin zmax])
   set(gca, 'LineWidth',   2, ...
            'Position',    [0.1500 0.1500 0.7350 0.7050], ...
            'FontWeight', 'bold', ...
            'FontSize',    16)
   colormap(hsv);
   view(-16, 35);
   %
   % --- Titles, labels, and other displayed info.
   %
   text('String',               LAMBDA, ...
        'Units',               'Normalized', ...
        'Position',             [0.780 1.07], ...
        'HorizontalAlignment', 'Left', ...
        'FontWeight',          'Bold', ...
        'FontUnits',           'Normalized', ...
        'FontSize',             0.06)
   text('String',               [' = ', num2str(lambda, '%5.3f')], ...
        'Units',               'Normalized', ...
        'Position',             [0.930 1.07], ...
        'HorizontalAlignment', 'Left', ...
        'FontWeight',          'Bold', ...
        'FontUnits',           'Normalized', ...
        'FontSize',             0.06)
   text('String',              'dx', ...
        'Units',               'Normalized', ...
        'Position',             [0.780 1.00], ...
        'HorizontalAlignment', 'Left', ...
        'FontWeight',          'Bold', ...
        'FontUnits',           'Normalized', ...
        'FontSize',             0.06)
   text('String',              ' = 2 \pi / N', ...
        'Units',               'Normalized', ...
        'Position',             [0.930 1.00], ...
        'HorizontalAlignment', 'Left', ...
        'FontWeight',          'Bold', ...
        'FontUnits',           'Normalized', ...
        'FontSize',             0.06)
   text('String',              'N', ...
        'Units',               'Normalized', ...
        'Position',             [0.780 0.93], ...
        'HorizontalAlignment', 'Left', ...
        'FontWeight',          'Bold', ...
        'FontUnits',           'Normalized', ...
        'FontSize',             0.06)
   text('String',               [' = ', num2str(N)], ...
        'Units',               'Normalized', ...
        'Position',             [0.930 0.93], ...
        'HorizontalAlignment', 'Left', ...
        'FontWeight',          'Bold', ...
        'FontUnits',           'Normalized', ...
        'FontSize',             0.06)
   text('String',               SCHtex, ...
        'Units',               'Normalized', ...
        'Position',             [-0.10 1.07], ...
        'HorizontalAlignment', 'Left', ...
        'FontWeight',          'Bold', ...
        'FontUnits',           'Normalized', ...
        'FontSize',             0.06)
   text('String',               EQNtex, ...
        'Units',               'Normalized', ...
        'Position',             [-0.10 0.02], ...
        'HorizontalAlignment', 'Left', ...
        'FontWeight',          'Bold', ...
        'FontUnits',           'Normalized', ...
        'FontSize',             0.06)
   xlabel('{\bf x}',  'FontSize', 22)
   ylabel('{\bf t}',  'FontSize', 22)
   zlabel('{\bf u}',  'FontSize',             22, ...
                      'HorizontalAlignment', 'Right', ...
                      'Rotation', 0)
end
fprintf('\n Rotate 3D is on in the figures.')
fprintf('\n Type "close all" to close the figures. \n \n')
% ============================================================================

function MakeTheSchemeFigure(n, N)
% ============================================================================
SSz = get(0, 'ScreenSize');
SW  = SSz(3);% ------------ Screen Width.
SH  = SSz(4);% ------------ Screen Height.
Sml = min(SW, SH);% ------- Screen minimum length.
% ------------------------- Lengths below in fractions of Sml.
dX = 0.03*n;% --- Figure window distance from screen right edge.
dY = 0.03*n;% --- Figure window distance from screen top   edge.
fW = 6.55*0.07;% --- Figure window width.
fH = 5.00*0.07;% --- Figure window height.
% ------------------------- Figure position.
POF = [SW - (fW + dX)*Sml, SH - (fH + dY)*Sml, fW*Sml, fH*Sml];
%
NOF = ['Solution with N = ', num2str(N), ' points in space.'];
%
figure('Name', NOF, 'NumberTitle', 'off', 'Position', POF)
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% EOF